// Copyright (C) 2006 Erik Dahlberg
// 
// This file is part of LSystem3D.
// 
// LSystem3D is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// LSystem3D is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with LSystem3D; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA




#ifndef TURTLE_H
#define TURTLE_H

#include <stack>

#include "coordinate.h"
#include "model.h"
#include "vector.h"

using namespace std;



/**
 * The model creator
 */
class Turtle
{
public:
    /**
     * Constructor
     * @param model create this model
     */
    Turtle(Model *model);

    /**
     * Destructor
     */
    ~Turtle();
    
    /**
     * Turn left
     */
    void turnLeft();
    
    /**
     * Turn right
     */
    void turnRight();
    
    /**
     * Pitch down
     */
    void pitchDown();
    
    /**
     * Pitch up
     */
    void pitchUp();
    
    /**
     * Roll left
     */
    void rollLeft();
    
    /**
     * Roll right
     */
    void rollRight();
    
    /**
     * Turn around
     */
    void turnAround();
    
    /**
     * Walk forward
     */
    void walk();
    
    /**
     * Walk forward, creating a filled surface
     */
    void fillWalk();
    
    /**
     * Save current state to stack
     */
    void push();
    
    /**
     * Restore old state from stack
     */
    void pop();
    
    /**
     * Reset to default state
     */
    void reset();
    
    /**
     * Set turn/pitch/roll angle
     * @param angle the angle, in radians
     */
    void setAngle(double radians);
    
    /**
     * Get turn/pitch/roll angle
     * @return the angle, in radians
     */
    double getAngle();
    
protected:
    
    Coordinate _position;       // Current position
    
    Vector _heading;    // Forward pointing vector
    Vector _left;       // Left pointing vector
    Vector _up;         // Up pointing vector
    
    double _angle;      // Turn/pitch/roll angle, in radians
    
    stack<Coordinate> _positionStack;       // Positions in stack
    stack<Vector> _headingStack;            // Heading vectors in stack
    stack<Vector> _leftStack;               // Left vectors in stack
    stack<Vector> _upStack;                 // Up vectors in stack
    stack<double> _diameterStack;           // Diameters in stack
    stack<int> _colorIndexStack;            // Colors indices in stack
    
    Model *_model;    // The created model
};



#endif
